#version 330 core

struct PointLight {
    vec3 position;  
  
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
	
    float constant;
    float linear;
    float quadratic;
};
#define NR_POINT_LIGHTS 1

struct Material {
    sampler2D diffuse;
    sampler2D specular;
	sampler2D normal;
	float shininess;
}; 

  
uniform vec3 camera_postion;
uniform PointLight point_light[NR_POINT_LIGHTS];
uniform Material material;
uniform samplerCube depth_map;


in vec3 Frag_pos;    
in vec2 Tex_Coords;
in mat3 TBN;

// This Color value is written to the first ColorBuffer
layout (location = 0) out vec4 Color;

// The second value is used for bloom and is written to the second ColorBuffer
layout (location = 1) out vec4 BrightColor;

float ShadowCalculation()
{
	float shadow_factor = 0.0;
   	float far_plane= 50.0;
	float bias = 0.15;

	vec3 sampling_pcf[20] = vec3[]
	(
	   vec3(1, 1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1, 1,  1), 
	   vec3(1, 1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
	   vec3(1, 1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1, 1,  0),
	   vec3(1, 0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1, 0, -1),
	   vec3(0, 1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0, 1, -1)
	);

    float view_to_frag_dist = length(camera_postion - Frag_pos);

	vec3 frag_to_light = Frag_pos - point_light[0].position;
   
    float current_depth = length(frag_to_light);
           
	float disk_radius = (1.0 + (view_to_frag_dist / far_plane)) / 25.0;
    for(int i = 0; i < sampling_pcf.length(); ++i)
    {
        float closest_depth = texture(depth_map, frag_to_light + sampling_pcf[i] * disk_radius).r;
        closest_depth *= far_plane;   
        if((current_depth - bias) > closest_depth)
            shadow_factor++;
    }
    shadow_factor = shadow_factor / float(sampling_pcf.length());
        
    return shadow_factor;
}

void main()
{
	vec3 normal = texture(material.normal, Tex_Coords).rgb;
	normal = normalize(normal * 2.0 - 1.0);   
	normal = normalize(TBN * normal);

	for(int i = 0; i < NR_POINT_LIGHTS; i++){
		float distance    = length(point_light[i].position - Frag_pos);
		float attenuation = 1.0 / (point_light[i].constant + point_light[i].linear * distance + point_light[i].quadratic * (distance * distance));
		
		vec3 ligh_direction = normalize(point_light[i].position - Frag_pos);
		vec3 view_direction = normalize(camera_postion - Frag_pos);
		vec3 reflection_direction= reflect(-ligh_direction, normal);

		vec3 ambient =  texture(material.diffuse, Tex_Coords).rgb * point_light[i].ambient;	
		vec3 diffuse = point_light[i].diffuse * max(dot(normal, ligh_direction), 0.0) * texture(material.diffuse, Tex_Coords).rgb;     
		vec3 specular = point_light[i].specular * pow(max(dot(view_direction, reflection_direction), 0.001), material.shininess) * texture(material.specular, Tex_Coords).rgb;
		
		float shadow_factor = ShadowCalculation();                      

		Color += vec4((ambient + (1.0 - shadow_factor) *(diffuse + specular)) *attenuation, 1.0);


		// if brightness is higher than certain value, write it to the brightness colorbuffer
		float brightness = (Color.x + Color.y + Color.z) / 3.0f;
		if(brightness >= 0.95f) {
			BrightColor = Color;
			}
		else {
			BrightColor = vec4(0.0f, 0.0f, 0.0f, 1.0f);
		}
	 }
} 